/*
 Revised ArduinoTARDIS using ATX/PWM power shield
 Refactored into separate header files.
 */

#define TARDIS_VERSION "$Id: ArduinoTARDIS.ino 17 2019-06-06 07:18:18Z cafelizardo $"

const int ATX_CTRL = 2;         // atx output control
const int SIGN_RED_LED = 3;     // PWM red output 
const int ATX_POWER_LIGHT =  4; // atx power on indicator
const int SIGN_GREEN_LED = 5;   // PWM green output
const int SIGN_BLUE_LED = 6;    // PWM blue output
const int IR_RECV_PIN = 46;     // IR receiver input 
const int RESERVED_09 = 9;      // PWM reserved
const int CABINET_LIGHT = 10;   // PWM lower cabinet lights
const int ROOF_LIGHT = 11;      // PWM roof light output
const int DOOR_SWITCH = 12;     // door trigger
const int INVERT_PIN = 38;      // invert flag - real or test board
const int DOOR_1 = 40;
const int DOOR_2 = 41;
const int DOOR_3 = 42;
const int DOOR_4 = 43;

#define ENABLE_NETWORK 1
#if ENABLE_NETWORK
#define WINC_CS  A3 
#define WINC_IRQ 7
#define WINC_RST A5
#define WINC_EN  -1     // or, tie EN to VCC
#endif // ENABLE_NETWORK


#include "ATX_Shield.h"         // atx power supply control
#include "Flasher.h"            // basic flasher class
#include "Pulser.h"             // breathing flasher
#include "RGB_Pulser.h"         // RGB breather flasher
#include "ir_decoder.h"         // IR decoder

#include "tardis_audio.h"       // audio using WavTrigger board


// Flasher ctor takes led, on perid and off period
Flasher builtin_led(LED_BUILTIN, 10, 1000); // flash power light at a steady interval

ATX_Shield atx_shield(ATX_CTRL);               // atx control pin

// Pulser ctor takes pin, interval, maxValue, initial, increment, offset
Pulser roof_light(ROOF_LIGHT, 5, 63, 20, 10, 0);
//Flasher roof_light(ROOF_LIGHT, 100, 50);

//Pulser cabinet_light(CABINET_LIGHT, 5, 63, 20, 10, 0);
Flasher cabinet_light(CABINET_LIGHT);

// RGB_Pulser ctor takes pins for red green and blue
RGB_Pulser sign_color(SIGN_RED_LED, SIGN_GREEN_LED, SIGN_BLUE_LED);

IR_Decoder ir_decoder;

// door state
int old_door_position = -1;
int door_position = -1;

// overrides via remote
int door_override = -1;
int roof_override = -1;
long sign_color_override = -1;


#define READY_TIME   5000   // 5 seconds
#define ON_TIME     60000   // 1 minute
long alarm_time = 0;

int sound_track = 0;
char buffer[100];

#if ENABLE_NETWORK
#include "tardis_network.h"     // network connection 

#else // ENABLE_NETWORK
#define WL_NO_SHIELD 255
int network_status = WL_NO_SHIELD;
#endif // ENABLE_NETWORK


// live or prototype
int invert = -1;

int getDoorPosition() {
    return 0
        +  1 * (digitalRead(DOOR_1) ^ (1 - invert))
        +  2 * (digitalRead(DOOR_2) ^ (1 - invert))
        +  4 * (digitalRead(DOOR_3) ^ (1 - invert))
        +  8 * (digitalRead(DOOR_4) ^ (1 - invert))
      //+ 16 * (digitalRead(DOOR_SWITCH) ^ (1 - invert))
        ;
}

// the setup routine runs once when you press reset:
void setup() {
  while (!Serial);
  Serial.begin(115200);

  Serial.println("");
  Serial.println(TARDIS_VERSION);
  Serial.println(ATX_SHIELD_VERSION);
  Serial.println(FLASHER_VERSION);
  Serial.println(PULSER_VERSION);
  Serial.println(RGB_PULSER_VERSION);
  Serial.println(IR_DECODER_VERSION);
  Serial.println(TARDIS_AUDIO_VERSION);
#if ENABLE_NETWORK
  Serial.println(TARDIS_NETWORK_VERSION);
#else // ENABLE_NETWORK
  Serial.println("Network not enabled");
#endif // ENABLE_NETWORK
  Serial.println("----");

  // check invert flag - real or test board
  pinMode(INVERT_PIN, INPUT_PULLUP);
  invert = digitalRead(INVERT_PIN);
  Serial.print("invert="); Serial.println(invert, DEC);

  // initialize ATX interface
  atx_shield.setup();

  // initialize inputs
  pinMode(DOOR_SWITCH, INPUT_PULLUP);
  pinMode(DOOR_1, INPUT_PULLUP);
  pinMode(DOOR_2, INPUT_PULLUP);
  pinMode(DOOR_3, INPUT_PULLUP);
  pinMode(DOOR_4, INPUT_PULLUP);

  pinMode(ATX_POWER_LIGHT, OUTPUT);

  // initialize LEDs
  sign_color.red()->setInterval(42).setInitial(0).setIncrement(3).setOffset(-20).setInvert(invert);
  sign_color.green()->setInterval(45).setInitial(-20).setIncrement(4).setOffset(-20).setInvert(invert);
  sign_color.blue()->setInterval(40).setInitial(30).setIncrement(5).setOffset(-10).setInvert(invert);

  // initialize ir decoder library
  ir_decoder.setup();

  // initialize sound effects
  TardisAudio::setup();

  old_door_position = getDoorPosition();

#if !ENABLE_NETWORK
  Serial.println("WiFi disabled.");
#else // ENABLE_NETWORK
  tardis_network_setup();
#endif // ENABLE_NETWORK

  Serial.println("====");
  
  alarm_time = millis() + READY_TIME;   // flash when reset

} // setup()

// the loop routine runs over and over again forever:
void loop() {

    door_position = getDoorPosition();
    if (door_position == old_door_position) door_override = -1;

#if ENABLE_NETWORK
    tardis_network_check();
#endif // ENABLE_NETWORK

    builtin_led.update();

    // check for IR commands
    static int old_ir_cmd = -1;
    int ir_cmd;
    //ir_cmd = ir_decoder.check();
    if (ir_cmd != old_ir_cmd) {
        sound_track = 0;
        switch (ir_cmd) {
            case 1: sound_track = Num_list.getNext();
                    break;
        } // switch
        if (sound_track > 0) {
            sprintf(buffer, "Playing sound_track %d", sound_track);
            Serial.println(buffer);
            TardisAudio::playTrack(sound_track);
        }
        old_ir_cmd = ir_cmd;
    }

    if (door_position != old_door_position) {

        int door_rate = (2 - door_position) * 100;
        builtin_led.setOnTime(door_rate).setOffTime(2 * door_rate);

        if (old_door_position != -1) {  // skip sound at startup
            sound_track = 0;
            if (TardisAudio::sound_bank == 0) {
                switch (door_position) {
                    case 0: sound_track = TARDIS_list.getNext();
                            roof_light.setInterval(5).setIncrement(10);
                            roof_override = -1;
                            sign_color_override = -1;
                            break;
                    default: sound_track = Theme_list.getNext();
                            roof_light.setInterval(1).setIncrement(5);
                            roof_override = -1;
                            sign_color_override = -1;
                            break;
                } // switch
            } else {
                sound_track = Alt_list.getNext();
            }
            if (sound_track > 0) {
                sprintf(buffer, "Playing sound_track %d", sound_track);
                Serial.println(buffer);
                TardisAudio::playTrack(sound_track);
            }
        }

        Serial.print("door_position="); Serial.print(door_position, DEC);
        Serial.println("");

        digitalWrite(ATX_POWER_LIGHT, HIGH);
        atx_shield.powerOn();
        sign_color.enable();
        roof_light.enable();
        cabinet_light.enable();

        old_door_position = door_position;

        long wait_time = ON_TIME * (door_position > 0 ? door_position : 1);
        alarm_time = millis() + wait_time;
        Serial.print("wait_time="); Serial.print(wait_time / 1000, DEC);
        Serial.print("  alarm_time="); Serial.print(alarm_time / 1000, DEC);
        Serial.println();

    } // door changed

    // while counting down
    if (alarm_time > 0) {

        long time_remaining = alarm_time - millis();

        if (time_remaining <= 0) {

            digitalWrite(ATX_POWER_LIGHT, LOW);
            atx_shield.powerOff();
            sign_color.disable();
            roof_light.disable();
            cabinet_light.disable();

            TardisAudio::stopAll();

            alarm_time = 0; // turn off alarm

        } else {

            if (!door_position) {

                // door closed - take off mode
                cabinet_light.disable();

                if (roof_override >= 0) {
                    roof_light.output(roof_override);
                } else {
                    roof_light.update();
                }

                if (sign_color_override >= 0) {
                    sign_color.output(sign_color_override);
                } else {
                    sign_color.update();
                }

            } else {

                // door open - display contents of cabinet
                cabinet_light.output(100);

                if (roof_override >= 0) {
                    roof_light.output(roof_override);
                } else {
                    roof_light.update();
                }

                if (sign_color_override >= 0) {
                    sign_color.output(sign_color_override);
                } else {
                    sign_color.output(0);
                }
            }
        }
    } // while counting down

    if (network_status == WL_NO_SHIELD)
        delay(50);

} // loop()

// ArduinoTARDIS.ino
